#素材
class ResourcePicture < ActiveRecord::Base
  belongs_to :artist
  belongs_to :license
  belongs_to :picture
  has_many :panel_pictures
  belongs_to :original_picture
  
  validates :ext, :presence => true, :length => {:maximum => 4}, :inclusion => {:in => ['png', 'jpeg', 'gif']}
  validates :width, :presence => true, :numericality => true, :natural_number => true
  validates :height, :presence => true, :numericality => true, :natural_number => true
  validates :filesize, :presence => true, :numericality => {:greater_than => 0, :less_than_or_equal_to => 2000000}, :natural_number => true
  validates :md5, :presence => true, :length => {:minimum => 32, :maximum => 32}
  validates :artist_id, :presence => true, :numericality => true, :existence => {:both => false}
  validates :license_id, :presence => true, :numericality => true, :existence => {:both => false}
  validates :original_picture_id, :presence => true, :numericality => true, :existence => {:both => false}
  validates :artist_name, :presence => true
  validates :classname, :presence => true, :length => {:maximum => 50}
  validates :picture_id, :presence => true, :numericality => true, :existence => {:both => false}
  
  before_validation :valid_encode
  
  def valid_encode
    ['artist_name', 'classname', 'credit', 'settings'].each do |a|
      next if attributes[a] == nil
      raise Pettanr::BadRequest unless attributes[a].valid_encoding?
    end
  end
  
  def supply_default
  end
  
  def overwrite op
    attr = {:width => op.width, :height => op.height, :ext => op.ext, :filesize => op.filesize, 
      :original_picture_id => op.id, :artist_id => op.artist_id, :md5 => op.md5
    }
    self.attributes = attr
  end
  
  def own? roles
    roles = [roles] unless roles.respond_to?(:each)
    ar = ResourcePicture.get_artist_from_roles roles
    return false unless ar
    self.artist_id == ar.id
  end
  
  def visible? roles
    if MagicNumber['run_mode'] == 0
      return false unless guest_role_check(roles)
    else
      return false unless resource_reader_role_check(roles)
    end
    true
  end
  
  def filename
    "#{self.id}.#{self.ext}"
  end
  
  def gifname
    "#{self.id}.gif"
  end
  
  def mime_type
    "image/#{self.ext}"
  end
  
  def url subdir = nil
    '/resource_pictures/' + (subdir.to_s.empty? ? '' : subdir.to_s + '/' ) + filename
  end
  
  def to_gif?
    self.ext == 'png' and self.flag_gif_convert >= 0
  end
  
  def thumbnail(imager)
    tw, th = ResourcePicture.fix_size_both(MagicNumber['thumbnail_width'], MagicNumber['thumbnail_height'], rimg.columns, rimg.rows)
    ResourcePicture.resize(rimg.to_blob, tw, th).to_blob
  end
  
  def tmb_opt_img_tag
    tw, th = PettanImager.thumbnail_size(self.width, self.height)
    {:src => self.url, :width => tw, :height => th}
  end
  
  def opt_img_tag
    {:src => self.url('full'), :width => self.width, :height => self.height}
  end
  
  def self.default_page_size
    25
  end
  
  def self.max_page_size
    100
  end
  
  def self.page prm = nil
    page = prm.to_i
    page = 1 if page < 1
    page
  end
  
  def self.page_size prm = self.default_page_size
    page_size = prm.to_i
    page_size = self.max_page_size if page_size > self.max_page_size
    page_size = self.default_page_size if page_size < 1
    page_size
  end
  
  def self.mylist_where ar
    ['resource_pictures.artist_id = ?', ar.id]
  end
  
  def self.himlist_where ar
    ['resource_pictures.artist_id = ?', ar.id]
  end
  
  def self.list page = 1, page_size = self.default_page_size
    ResourcePicture.includes(ResourcePicture.list_opt).order('resource_pictures.updated_at desc').offset((page -1) * page_size).limit(page_size)
  end
  
  def self.mylist ar, page = 1, page_size = Author.default_resource_picture_page_size
    ResourcePicture.where(self.mylist_where(ar)).includes(ResourcePicture.list_opt).order('resource_pictures.updated_at desc').offset((page -1) * page_size).limit(page_size)
  end
  
  def self.himlist ar, page = 1, page_size = Author.default_resource_picture_page_size
    ResourcePicture.where(self.himlist_where(ar)).includes(ResourcePicture.list_opt).order('resource_pictures.updated_at desc').offset((page -1) * page_size).limit(page_size)
  end
  
  def self.list_paginate page = 1, page_size = self.default_page_size
    Kaminari.paginate_array(Array.new(ResourcePicture.count, nil)).page(page).per(page_size)
  end
  
  def self.mylist_paginate ar, page = 1, page_size = Author.default_resource_picture_page_size
    Kaminari.paginate_array(Array.new(ResourcePicture.where(self.mylist_where(ar)).count, nil)).page(page).per(page_size)
  end
  
  def self.himlist_paginate ar, page = 1, page_size = Author.default_resource_picture_page_size
    Kaminari.paginate_array(Array.new(ResourcePicture.where(self.himlist_where(ar)).count, nil)).page(page).per(page_size)
  end
  
  def self.list_opt
    {:license => {}, :artist => {}, :picture => {} }
  end
  
  def self.list_json_opt
    {:include => {:license => {}, :artist => {}, :picture => {}} }
  end
  
  def self.show rid, roles
    opt = {}
    opt.merge!(self.show_opt)
    r = ResourcePicture.find(rid, opt)
    raise ActiveRecord::Forbidden unless r.visible?(roles)
    r
  end
  
  def self.show_opt
    {:include => {:license => {}, :artist => {}, :picture => {}} }
  end
  
  def self.show_json_opt
    {:include => {:license => {}, :artist => {}, :picture => {}} }
  end
  
  def self.edit rid, ar
    opt = {}
    opt.merge!(self.show_opt)
    r = ResourcePicture.find(rid, opt)
    raise ActiveRecord::Forbidden unless r.own?(ar)
    r
  end
  
  def new_picture imager
    pc = Picture.new
    pc.supply_default
    pc.overwrite self
    r = pc.store imager
    return pc if r
    self.errors.add :base, Picture.model_name.human + I18n.t('errors.not_create')
    false
  end
  
  def store imager
    return false unless imager
    res = false
    self.overwrite self.original_picture
    ResourcePicture.transaction do
      self.original_picture.published_at = Time.now
      self.original_picture.stopped_at = nil
      raise ActiveRecord::Rollback unless self.original_picture.save
      pc = self.new_picture imager
      if pc
        self.picture_id = pc.id
        if res = self.save
          res = self.store_picture_with_gif(imager)
        end
      else
      end
      raise ActiveRecord::Rollback unless res
    end
    res
  end
  
  def store_picture_with_gif(imager)
    if res = self.store_picture(imager, self.filename)
      if self.to_gif?
        if gifimager = imager.to_gif
          if res = self.store_picture(gifimager, self.gifname)
          else
            self.errors.add :base, I18n.t('picture_io.error')
          end
        else
          self.errors.add :base, I18n.t('errors.not_convert')
          res = false
        end
      end
    else
      self.errors.add :base, I18n.t('picture_io.error')
    end
    res
  end
  
  def store_picture(imager, fn)
    res = false
    thumbnail_imager = self.flag_thumbnail >= 0 ? imager.to_thumbnail : imager
    return false unless thumbnail_imager
    begin
      PictureIO.resource_picture_io.put(thumbnail_imager.binary, fn)
      PictureIO.resource_picture_io.put(imager.binary, fn, 'full')
      res = true
    rescue PictureIO::Error
      res = false
    end
    res
  end
  
  def restore(subdir = nil)
    PictureIO.resource_picture_io.get self.filename, subdir
  end
  
  def unpublish
    res = false
    ResourcePicture.transaction do
      self.original_picture.published_at = nil
      self.original_picture.stopped_at = Time.now
      raise ActiveRecord::Rollback unless self.original_picture.save
      begin
        PictureIO.resource_picture_io.delete(self.filename) if PictureIO.resource_picture_io.exist?(self.filename)
        PictureIO.resource_picture_io.delete(self.filename, 'full') if PictureIO.resource_picture_io.exist?(self.filename, 'full')
      rescue PictureIO::Error
        res = false
        raise ActiveRecord::Rollback
      end
      res = self.destroy
      raise ActiveRecord::Rollback unless res
    end
    res
  end
  
  def self.visible_count
    ResourcePicture.count
  end
  
  def picture_data
    Base64.encode64(self.restore 'full')
  end
  
  def credit_template
    "#{self.classname.tableize}/attributes/credit"
  end
  
  def full_credit_template
    "#{self.classname.tableize}/attributes/full_credit"
  end
  
  def credit_data
    begin
      @credit_data = JSON.parse(self.credit) unless @credit_data
    rescue 
    end
    @credit_data
  end
  
  def flags
    begin
      @flags = JSON.parse(self.settings) unless @flags
    rescue 
    end
    @flags
  end
  
  def flags=(s)
    @flags = s
  end
  
  def flag_open
    @flag_open = flags["open"] unless @flag_open
    @flag_open
  end
  
  def flag_commercial
    @flag_commercial = flags["commercial"] unless @flag_commercial
    @flag_commercial
  end
  
  def flag_official
    @flag_official = flags["official"] unless @flag_official
    @flag_official
  end
  
  def flag_attribution
    @flag_attribution = flags["attribution"] unless @flag_attribution
    @flag_attribution
  end
  
  def flag_derive
    @flag_derive = flags["derive"] unless @flag_derive
    @flag_derive
  end
  
  def flag_thumbnail
    @flag_thumbnail = flags["thumbnail"] unless @flag_thumbnail
    @flag_thumbnail
  end
  
  def flag_gif_convert
    @flag_gif_convert = flags["gif_convert"] unless @flag_gif_convert
    @flag_gif_convert
  end
  
  def flag_reverse
    @flag_reverse = flags["reverse"] unless @flag_reverse
    @flag_reverse
  end
  
  def flag_resize
    @flag_resize = flags["resize"] unless @flag_resize
    @flag_resize
  end
  
  def flag_sync_vh
    @flag_sync_vh = flags["sync_vh"] unless @flag_sync_vh
    @flag_sync_vh
  end
  
  def flag_overlap
    @flag_overlap = flags["overlap"] unless @flag_overlap
    @flag_overlap
  end
  
end
